@@ -21,6 +21,7 @@
 #           2025 Andrew Bower <andrew@bowerg.uk>
 #
 use strict;
+use utf8;
 use Getopt::Long qw(:config no_ignore_case);
 use Pod::Usage;
 use File::Basename;
@@ -46,7 +47,7 @@ use constant {
 };
 
 my $OURNAME = "sysv-rc-conf";
-my $VERSION = "1.2.1";
+my $VERSION = "1.3.0";
 my $called_as_chkconfig = $0 =~ /chkconfig$/;
 
 # Runlevel patterns: [ state regex, inverse video, colour, summary ]
@@ -128,13 +129,15 @@ my %opts = (
     defdiff     => 1,
     pictograms  => 0,
     warninit    => 1,
+    colour      => (defined $ENV{NO_COLOR}) ? 0 : 1,
+    classic     => 0,
 );
 my @mutually_exclusive_options = qw(chkcfg_list chkcfg_add chkcfg_del);
 
 foreach my $conffile (grep { -s } (
     "/usr/share/sysv-rc-conf/config",
     "/etc/sysv-rc-conf.conf",
-    "$ENV{HOME}/.sysv-rc-confrc",
+    ($ENV{XDG_CONFIG_HOME} // "$ENV{HOME}/.config")."/sysv-rc-conf.conf",
 )) {
     if (open (my $conf, "<", $conffile)) {
         while (<$conf>) {
@@ -174,8 +177,11 @@ GetOptions("cache=s"	=> \$opts{cache_dir},
            "no-interpret"   => sub { $opts{interpret} = 0; },
            "pictograms"     =>      \$opts{pictograms},
            "no-pictograms"  => sub { $opts{pictograms} = 0; },
+           "colour"         =>      \$opts{colour},
+           "no-colour"      => sub { $opts{colour} = 0; },
            "defdiff"        =>      \$opts{defdiff},
            "no-defdiff"     => sub { $opts{defdiff} = 0; },
+           "classic"        =>      \$opts{classic},
 	  ) or exit(1);
 
 $opts{cache_dir} //= "$opts{root}var/lib/sysv-rc-conf";
@@ -192,6 +198,9 @@ $opts{chkcfg_type} eq "sysv" or die "Only 'sysv' type services supported";
 $opts{pictograms} = ($ENV{LANG} =~ /\.UTF-8$/ ? 1 : 0) if $opts{pictograms};
 $opts{ui} //= 0 if $opts{purge_cache};
 $opts{ui} //= 1;
+if ($opts{classic}) {
+    $opts{interpret} = $opts{pictograms} = $opts{defdiff} = $opts{colour} = 0;
+}
 
 my $etc_rc = $opts{root} . "etc/rc";
 my $initd = $opts{root} . "etc/init.d";
@@ -247,17 +256,21 @@ if (defined $opts{chkcfg_list} or
 
 # CUI Globals
 my $mode_text = $opts{read_only} ? "RO" : "";
-my $title_help    = "SysV Runlevel Config   +/-: start/stop service now  h: help  q: quit  $mode_text";
+my $title_help    = "SysV Runlevel Config   +/-: start/stop   i: info   h: help   q: quit  $mode_text";
 my $title_helpscr = "SysV Runlevel Config                                h/q: quit help    $mode_text";
 my $cui;
 my @snames_per_screen;
 my %box_pos;
 my %service_controls = ();
 my %extra_controls = ();
+my $last_service = undef;
+my $last_page = undef;
+my @found = ();
+my $found_index;
 
 if ($opts{ui}) {
     $cui = new Curses::UI( -clear_on_exit    => 0,
-			      -color_support    => 1,
+			      -color_support    => $opts{colour},
 			      -default_colors   => 1,
 			    ) or die "Can't create base Curses::UI object";
     # Get the service names for each screen
@@ -266,9 +279,8 @@ if ($opts{ui}) {
 
     create_bottom_box();
     create_main_window( \%service_controls );
-    $cui->set_binding( \&toggle_help,    "h" );
     $cui->set_binding( \&revert_changes, "r" );
-    $s[$current_screen]->focus();
+    first_service();
 }
 
 ui_warning("$unsuitable_init") if defined $unsuitable_init and $opts{warninit};
@@ -277,7 +289,7 @@ if ($opts{ui}) {
     $cui->mainloop();
 }
 
-if ($opts{insserv} and not $opts{read_only} and not $passive_ops_only) {
+if ($opts{insserv} and not $opts{priority} and not $opts{read_only} and not $passive_ops_only) {
     my $insserv = "/usr/sbin/insserv";
 
     if ( -x $insserv ) {
@@ -535,6 +547,44 @@ sub revert_one
     $widget->focus();
 }
 
+sub find_dialog
+{
+    my $search_string = $cui->question(-question =>
+        'Find service. (Then press n/p for next/previous.)');
+    if (($search_string // "") =~ /.+/) {
+        @found = sort grep { index($_, $search_string) != -1 } (keys %services);
+        undef $found_index;
+    }
+    goto_next();
+}
+
+sub goto_service
+{
+    my ($sn) = @_;
+    my $box = $service_controls{$sn}->{$show_rls[0]};
+    my $scr = $box->parent();
+    my $id = $box->userdata()->{id};
+    $current_screen = first { $s[$_] == $scr } 0..$#s;
+    $id =~ /^(\d+),(\d+)$/;
+    _move_focus(int($1), $box_pos{col});
+}
+
+sub goto_prev
+{
+    return if (scalar @found) == 0;
+    $found_index = ($found_index // 0) -1;
+    $found_index = (scalar @found) -1 if $found_index < 0;
+    goto_service($found[$found_index]);
+}
+
+sub goto_next
+{
+    return if (scalar @found) == 0;
+    $found_index = ($found_index // -1) + 1;
+    $found_index = 0 if $found_index == (scalar @found);
+    goto_service($found[$found_index]);
+}
+
 sub start_service
 {
     my ($widget) = @_;
@@ -646,6 +696,110 @@ sub set_lsb_defaults
     $widget->focus();
 }
 
+sub infobox
+{
+    my ($widget) = @_;
+    my $ud = $widget->userdata();
+    my $sn = $ud->{sn};
+    my $rl_state = rl_str($sn);
+    my ($invert, $colour, $note) = interp($sn, $rl_state);
+
+    my $lsb = $services{$sn}->{'lsb_info'};
+    if (!defined $lsb) {
+	$cui->error("No LSB headers defined for service: $sn\n");
+	return;
+    }
+    my $lsb_rls = $services{$sn}->{lsb_rls};
+
+    my $box = $cui->add(
+        "infobox", 'Window',
+        -title        => "$sn: $lsb->{'Short-Description'}",
+        -border       => 1,
+        -padtop       => 3,
+        -padbottom    => 1,
+        -padleft      => 12,
+        -padright     => 12,
+        -ipad         => 1,
+        -ipadbottom   => 0,
+        -modal        => 1,
+    );
+    $box->set_color_tfg($colour);
+    my $desc = $box->add(
+        undef, 'TextViewer',
+        -text         => $lsb->{'Description'} // $lsb->{'Short-Description'} // $sn,
+        -wrapping     => 1,
+        -height       => 4,
+        -vscrollbar   => 1,
+        -border       => 0,
+    );
+    my $grid = $box->add(
+        undef, "Container",
+        -releasefocus => 1,
+        -y            => 5,
+    );
+    my $label1 = $grid->add(
+        undef, "Label",
+        -text   => "Settings  :\nRunlevels :\nChanged?  :"
+    );
+    my $settings = $grid->add(
+        undef, 'Label',
+        -x       => 12,
+        -width   => 28,
+        -text    => $note,
+        -reverse => $invert,
+    );
+    $grid->add(
+        undef, 'Label',
+        -x      => 12,
+        -y      => 1,
+        -height => 2,
+        -text   => (join "\n",
+          (join ' ', (map { "$_".(rl_state($sn, $_) =~ tr/012/KS~/r) } qw( 1 2 3 4 5 0 6 S ))),
+          ($lsb_rls eq rl_str($sn) ? "Using LSB defaults" : "Changed from LSB defaults"),
+        ),
+    );
+    $settings->set_color_fg($colour);
+    my $code = "";
+    if (open (my $SCRIPT, "<:encoding(utf8)", "$initd/$sn")) {
+        chomp (my @lines = <$SCRIPT>);
+        close $SCRIPT;
+        my $width = length (sprintf "%d", $#lines);
+        my $ln = 1;
+        $code = join "\n", (map { sprintf "%0*d: %s", ($width, $ln++, $_); } @lines);
+    }
+    my $script = $grid->add(
+        undef, 'TextViewer',
+        -y      => 3,
+        -height => $grid->canvasheight() - 4,
+        -vscrollbar => 1,
+        -border     => 1,
+        -text       => $code,
+    );
+    $desc->set_color_sfg($colour);
+    $script->set_color_sfg($colour);
+    $grid->layout();
+
+    my $buttons = $box->add(
+        undef, 'Buttonbox',
+        -buttons      => [ { -label => 'OK', -shortcut => 'o',
+                             -onpress => sub { $cui->delete("infobox"); $cui->draw(); } } ],
+        -buttonalignment => 'middle',
+        -y      => $box->canvasheight() - 1,
+    );
+    $buttons->set_color_fg($colour);
+    $box->focus();
+    $script->focus();
+    sub exitbox {
+        $cui->delete("infobox");
+        $cui->draw();
+    };
+    $box->set_binding( \&exitbox,  "q" );
+    $box->set_binding( \&exitbox,  "o" );
+    $box->set_binding( \&exitbox,  "i" );
+    $box->set_binding( \&exitbox,  KEY_ENTER );
+    $buttons->draw();
+}
+
 sub get_pri_cache
 {
     my ($sn, $rl, $sk) = @_;
@@ -968,6 +1122,18 @@ sub current_runlevel
     }
 }
 
+sub default_runlevel
+{
+    my $rl = 2;
+    if (open (my $inittab, "<", "$opts{root}etc/inittab")) {
+        while (<$inittab>) {
+            $rl = $1 if /^\s*[^#:]+:([^:]+):initdefault:/;
+        }
+        close($inittab);
+    }
+    return $rl;
+}
+
 sub probably_oneshot
 {
     my $service = shift;
@@ -1159,29 +1325,32 @@ sub create_main_window
         $screen->set_binding( \&next_page, KEY_NPAGE );
         $screen->set_binding( \&prev_page, KEY_PPAGE );
         $screen->set_binding( \&first_service, KEY_HOME );
+        $screen->set_binding( \&last_service, KEY_END );
 	$screen->set_binding( sub { $cui->mainloopExit },  "q" );
+        $screen->set_binding( \&toggle_help,  "h" );
+        $screen->set_binding( \&find_dialog,  "/" );
+        $screen->set_binding( \&goto_next,    "n" );
+        $screen->set_binding( \&goto_prev,    "p" );
 
 	create_top_label($screen, $i, $#snames_per_screen);
 
 	my $left_label = '';
 	for (my $i = 0; $i < scalar(@$services); $i++) {
-	    $left_label .= $services->[$i] . "\n";
-	    if ($services->[$i] =~ /^\d\d (.+)$/) {
-		# If the labels had numbers, we don't need them anymore. 
-		$services->[$i] = $1;
-	    }
+            my $sn = $services->[$i];
+            $sn =~ s/^\d\d (.+)$/$1/;
+            $screen->add(
+                "label:$sn", 'Label',
+                -text   => $services->[$i],
+                -y      => TOP_LABEL_HEIGHT + $i,
+                -width  => $opts{label_width},
+                -height => 1,
+	    );
+            # If the labels had numbers, we don't need them anymore.
+            $services->[$i] = $sn;
 	}
 
 	my $row = TOP_LABEL_HEIGHT;
 
-	$screen->add(
-	    undef, 'Label',
-	    -text   => $left_label,
-	    -y	    => $row,
-	    -width  => $opts{label_width},
-	    -height => last_row() + 1,
-	);
-
 	foreach my $sn (@$services) {
 	    my $controls;
 	    if   ($opts{priority}) {
@@ -1208,10 +1377,15 @@ Quick key reference:
   PgDn or ^n: Next Page
   PgUp or ^p: Previous Page
   Home:       Go to first service
+  End:        Go to last service
   Backspace:  Restore service's symlinks back to state when started
   r:          Restore all symlinks back to state when sysv-rc-conf started
   -:          Stop service now
   +:          Start service now
+  i:          Show information about service
+  /:          Find service
+  n:           - go to next found service
+  p:           - go to previous found service
   h:          Toggle help screen on / off
   q:          Quit
 
@@ -1238,6 +1412,9 @@ Key to symbols:
   [ ]   A stop ('K') symlink is present for the service at this runlevel
   [~]   No symlink is present for the service at this runlevel
 
+        The current runlevel is shown in bold face.
+        The default runlevel is underlined.
+
   NOTE: When using either GUI layout (checkbox or priority), all
   configuration changes to the symlinks will happen IMMEDIATELY, not
   when the program exits.
@@ -1282,6 +1459,7 @@ EOF
     );
 
     $hw->set_binding( \&toggle_help,  "q" );
+    $hw->set_binding( \&toggle_help,  "h" );
 
 }
 
@@ -1367,6 +1545,7 @@ sub draw_simple_layout
 	$box->set_binding( \&toggle_multiuser, "m" );
 	$box->set_binding( \&set_lsb_defaults, "d" );
         $box->set_binding( \&revert_one,       KEY_BACKSPACE);
+        $box->set_binding( \&infobox, "i" );
 	$controls{$rl} = $box;
     }
     my $rlstates = rl_str($sn);
@@ -1431,40 +1610,74 @@ sub draw_priority_layout
 
 	$box->set_binding( \&start_service, "=", "+" );
 	$box->set_binding( \&stop_service,  "-" );
+        $box->set_binding( \&infobox, "i" );
 	$controls{$rl} = $box;
     }
     return \%controls;
 }
 
+sub rl_colour {
+    my ($rl) = @_;
+    if ($rl =~ /[06]/) {
+      return 'magenta';
+    } elsif ($rl =~ /[2345]/) {
+      return 'green';
+    } elsif ($rl =~ /1/) {
+      return 'yellow';
+    } elsif ($rl =~ /S/) {
+      return 'cyan';
+    } else {
+      return undef;
+    }
+}
+
 sub create_top_label
 {
     my ($window, $page, $of) = @_;
-
     my @label_rls = @show_rls;
-
     my $space = $opts{label_width} - 4;
     my $pages = sprintf(" %s%d/%d%s  ", $space > 13 ? "page " : "", $page + 1, $of + 1, $space > 10 ? "  " : "");
-    my $text = 'service' . sprintf("%*.*s", $space, $space, $pages) . shift @label_rls;
-    foreach (@label_rls) { $text .= "       $_" };
-    $text .= "\n";
-    $text .= "-" x ($opts{label_width} + 2 + scalar(@show_rls) * 8);
 
     $window->add(
 	undef, 'Label',
-	-text	=> $text,
+	-text	=> 'service' . sprintf("%*.*s", $space, $space, $pages),
 	-y	=> 0,
 	-x	=> 0,
-	-height	=> TOP_LABEL_HEIGHT,
+	-height	=> 1,
 	-textalignment	=> 'left',
     );
+    $window->add(
+	undef, 'Label',
+	-text	=> "-" x ($opts{label_width} + 2 + scalar(@show_rls) * 8),
+	-y	=> 1,
+	-x	=> 0,
+	-height	=> TOP_LABEL_HEIGHT - 1,
+	-textalignment	=> 'left',
+    );
+
+    my $rl = current_runlevel();
+    my $drl = default_runlevel();
+    my $x = $opts{label_width} + 3;
+    foreach (@label_rls) {
+        $window->add(
+            undef, 'Label',
+            -text       => $_,
+            -bold       => $_ eq $rl,
+            -underline  => $_ eq $drl,
+            -x          => $x,
+            -y          => 0,
+            #-fg         => $_ !~ /uncoloured/ ? rl_colour($_) : undef,
+        );
+        $x += 8;
+    }
 }
 
 sub create_bottom_box
 {
     my $cmd_text = '';
-    my $extra_controls = "m: toggle multiuser runlevels  d: defaults";
+    my $extra_controls = "m: toggle multiuser   /:find   d: defaults";
     my $cursor = $opts{pictograms} ?
-	"\x{2190}\x{2191}\x{2193}\x{2192}\x{21de}\x{21df}\x{21f1}\x{1f5b1}" :
+	"\x{2190}\x{2191}\x{2193}\x{2192}\x{21de}\x{21df}\x{21f1}\x{21f2}\x{1f5b1}" :
 	"<arrows>";
     if ($opts{priority}) {
 	$cmd_text = 
@@ -1489,7 +1702,7 @@ sub create_bottom_box
 	-width	=> -1,
 	-height => BOTTOM_LAB_HEIGHT,
 	-text   => $opts{pictograms} ?
-"$cursor   $extra_controls  r/\x{232b}: restore all/one\n$cmd_text" :
+"$cursor  $extra_controls  r/\x{232b}: restore all/one\n$cmd_text" :
 "$cursor  $extra_controls  r/BS: restore all/one\n$cmd_text",
     );
 }
@@ -1531,12 +1744,20 @@ sub prev_page
 
 sub first_service
 {
-    $current_screen = 0;
+    $current_screen = first_screen();
     verbose("Going to first service");
     $box_pos{row} = (0);
     _move_focus($box_pos{row}, $box_pos{col});
 }
 
+sub last_service
+{
+    $current_screen = last_screen();
+    verbose("Going to last service");
+    $box_pos{row} = last_row();
+    _move_focus($box_pos{row}, $box_pos{col});
+}
+
 sub got_focus
 {
     my $widget = shift;
@@ -1544,6 +1765,8 @@ sub got_focus
     # in userdata on creation and fetching it?
     my $userdata = $widget->userdata();
     my $id = $userdata->{id};
+    $last_service = $userdata->{sn};
+    $last_page = $widget->parent();
 
     $id =~ /^(\d+),(\d+)$/;
 
@@ -1582,7 +1805,17 @@ sub _move_focus
 {
     $box_pos{row} = $_[0];
     $box_pos{col} = $_[1];
+
+    # Underline current service
     my $box = $s[$current_screen]->getobj(cell_id($_[0], $_[1]));
+    my $sn = $box->userdata()->{sn};
+    my $label = $box->parent()->getobj("label:$sn");
+    if (defined $last_service) {
+        my $oldlabel = $last_page->getobj("label:$last_service");
+        $oldlabel->underline(0) if defined $oldlabel;
+    }
+    $label->underline(1) if defined $label;
+
     $box->focus();
 }
 
@@ -1756,6 +1989,14 @@ current runlevel settings differ from LSB defaults.
 
 Show, or do not show, unicode pictogram characters in the user interface.
 
+=item B<--colour>, B<--no-colour>
+
+Enable (default) or disable colour in the user interface.
+
+=item B<--classic>
+
+Equivalent to B<--no-colour --no-pictograms --no-interpret --no-defdiff>.
+
 =back
 
 =head2 CLI-Related Options
@@ -1921,6 +2162,14 @@ Whether to show symlink status analysis on right hand side.
 
 Disable warnings about alternative init systems.
 
+=item B<colour> or B<no-colour>
+
+Whether or not to use colour in the user interface.
+
+=item B<classic>
+
+Whether to turn off all fancy additions to the UI.
+
 =back
 
 =item F</usr/share/sysv-rc-conf/config>
@@ -1929,7 +2178,7 @@ If present, processed before F</etc/sysv-rc-conf.conf>. This is a good place
 for downstream packagers to put distribution-specific defaults that may differ
 from built-in defaults.
 
-=item F<.sysv-rc-confrc> in user's home directory
+=item F<.config/sysv-rc-conf.conf> in user's home directory
 
 If present, processed after F</etc/sysv-rc-conf.conf>.
 
